package pps.core.prediction.service;

import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.IBaseEnergyStorageCloudService;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_energy_storage_device.DynamicQueryBaseEnergyStorageDeviceInput;
import pps.cloud.base.service.data.base_energy_storage_device.DynamicQueryBaseEnergyStorageDeviceOutput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.prediction.entity.ThirdCurrentWellConditionEnt;
import pps.core.prediction.mapper.ThirdCurrentWellConditionMapper;
import pps.core.prediction.service.data.home_page.GetOverviewViewOutput;
import pps.core.prediction.service.data.home_page.GetStationViewInput;
import pps.core.prediction.service.data.home_page.GetStationViewOutput;
import pps.core.prediction.service.data.home_page.GetWellOverviewViewOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 首页
 *
 * @author ZWT
 * @date 2024/03/25 17:54
 */
@XText("首页模块")
@XService
public class HomePageService {

    /**
     * 首页模块--井场列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetStationViewOutput}>
     */
    @XApiAnonymous
    @XText("首页模块--井场列表")
    @XApiGet
    public XListResult<GetStationViewOutput> getStationList(XContext context, GetStationViewInput input) {
        List<GetStationViewOutput> outputs;
        String stationId = input.getStationId();
        if (CharSequenceUtil.isBlank(stationId)) {
            //查组织
            SysOrganizationCloudService service = context.getBean(SysOrganizationCloudService.class);
            XListResult<GetSysOrganizationViewOutput> result = service.getPlantList(context);
            result.throwIfFail();
            outputs = new ArrayList<>(result.getResult().size());
            result.getResult().forEach(item -> {
                GetStationViewOutput output = new GetStationViewOutput();
                output.setStationId(item.getId());
                output.setStationName(item.getOuName());
                outputs.add(output);
            });
        } else {
            //查电站
            List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPlantList(context, stationId);
            outputs = new ArrayList<>(plantList.size());
            plantList.forEach(item -> {
                GetStationViewOutput output = new GetStationViewOutput();
                output.setStationId(item.getId());
                output.setStationName(item.getStationName());
                outputs.add(GetStationViewOutput.builder()
                        .stationId(item.getId())
                        .stationName(item.getStationName())
                        .longitude(item.getLongitude())
                        .latitude(item.getLatitude())
                        .build());
            });
        }
        return XListResult.success(outputs);
    }

    /**
     * 首页模块--总览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetOverviewViewOutput}>
     */
    @XApiAnonymous
    @XText("首页模块--总览")
    @XApiGet
    public XSingleResult<GetOverviewViewOutput> getOverview(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellList(context, stationId);
        List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPlantList(context, stationId);
        List<DynamicQueryBasePowerLineOutput> lineList = this.getLineList(context, stationId);
        List<DynamicQueryBaseEnergyStorageDeviceOutput> storageList = this.getStorageList(context, stationId);
        return XSingleResult.success(GetOverviewViewOutput.builder()
                .wellNumber(wellList.size())
                .stationNumber(plantList.size())
                .lineNumber(lineList.size())
                .operatingLoad(wellList.stream()
                        .map(DynamicQueryBaseWellheadOutput::getServiceRating)
                        .reduce(BigDecimal.ZERO, BigDecimal::add))
                .photovoltaicPower(plantList.stream()
                        .map(GetBasePhotovoltaicPlantCloudOutput::getTotalPower)
                        .reduce(BigDecimal.ZERO, BigDecimal::add))
                .storagePower(storageList.stream()
                        .map(DynamicQueryBaseEnergyStorageDeviceOutput::getApparatusCapacity)
                        .reduce(BigDecimal.ZERO, BigDecimal::add))
                .build());
    }

    /**
     * 首页模块--井场实时监控
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetWellOverviewViewOutput}>
     */
    @XApiAnonymous
    @XText("首页模块--井场实时监控")
    @XApiGet
    public XSingleResult<GetWellOverviewViewOutput> getWellOverview(XContext context, GetStationViewInput input) {
        String stationId = input.getStationId();
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellList(context, stationId);
        ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
        List<ThirdCurrentWellConditionEnt> thirdCurrentWellList = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                .in(ThirdCurrentWellConditionEnt::getWellNumber, wellList.stream()
                        .map(DynamicQueryBaseWellheadOutput::getWellNumber)
                        .collect(Collectors.toList()))
        );
        int openWellNumber = 0;
        int stopWellNumber = 0;
        return XSingleResult.success(GetWellOverviewViewOutput.builder()
                .openWellNumber(openWellNumber)
                .stopWellNumber(stopWellNumber)
                .build());
    }



    /*------------------------------------------------- private ---------------------------------------------------------*/

    /**
     * 获取井口列表
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @return {@link List}<{@link DynamicQueryBaseWellheadOutput}>
     */
    private List<DynamicQueryBaseWellheadOutput> getWellList(XContext context, String stationId) {
        IBaseWellheadCloudService service = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = service.queryBaseWellheadListByParam(context, DynamicQueryBaseWellheadInput.builder()
                .ouId(stationId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 光伏电站列表
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @return {@link List}<{@link GetBasePhotovoltaicPlantCloudOutput}>
     */
    private List<GetBasePhotovoltaicPlantCloudOutput> getPlantList(XContext context, String stationId) {
        BasePhotovoltaicPlantCloudService service = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = service.getBasePhotovoltaicPlantList(context, GetBasePhotovoltaicPlantCloudInput.builder()
                .ouId(stationId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 输电线路列表
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getLineList(XContext context, String stationId) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> result = service.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .ouId(stationId)
                .lineTypeKey("10")
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取储能列表
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @return {@link List}<{@link DynamicQueryBaseEnergyStorageDeviceOutput}>
     */
    private List<DynamicQueryBaseEnergyStorageDeviceOutput> getStorageList(XContext context, String stationId) {
        IBaseEnergyStorageCloudService service = context.getBean(IBaseEnergyStorageCloudService.class);
        XListResult<DynamicQueryBaseEnergyStorageDeviceOutput> result = service.queryBaseStorageListByParam(context, DynamicQueryBaseEnergyStorageDeviceInput.builder()
                .ouId(stationId)
                .build());
        result.throwIfFail();
        return result.getResult();
    }
}